
#include "FileWriter.h"

// constructor
FileWriter::FileWriter ()
{
	_bApp = false;
	_bFileOpened = false;
}

// destructor
FileWriter::~FileWriter ()
{
	Close ();
}

// opens given ascii or unicode file for output
bool FileWriter::Open (const char* cFileNamePtr, CODE charSet, bool bAppend)
{
	// check if invaid file name
	if (cFileNamePtr == NULL || strcmp(cFileNamePtr, "") == 0)
		return false;

	// close previously opened file
	if (_bFileOpened)
		Close ();

	bool bExists = false;
	_bApp = bAppend;
	if (!_bApp)
		_ofsWriter.open (cFileNamePtr, ios::out|ios::binary);
	else
	{	// open in append mode
		bExists = DoesExist (cFileNamePtr);

		if (bExists)
		{
			_ofsWriter.open (cFileNamePtr, ios::in|ios::out|ios::binary|ios::ate);
			if (!_ofsWriter) {	Close();	return false;	}
			// if file opened APPEND, then characterSet must match existing 
			// file's codepage
			Word ucs_2_bom = 0;
			DWord ucs_4_bom = 0;
			wchar_t ucs_x_bom = 0;

			Byte* ucs_x_bom_ptr = ( (__ULE_UCS_SIZE == __ULE_UCS_2) ? 
					(Byte*)(&ucs_2_bom) : (Byte*)(&ucs_4_bom) );

			_ofsWriter.seekg(0, ios::beg);
			_ofsWriter.read ((char*)ucs_x_bom_ptr, __ULE_UCS_SIZE);

			ucs_x_bom = ( (__ULE_UCS_SIZE == __ULE_UCS_2) ? 
					(wchar_t)(*(Word*)ucs_x_bom_ptr) : (wchar_t)(*(DWord*)ucs_x_bom_ptr) );

			if ((ucs_x_bom == __ULE_UCS_BOM && charSet != UNI) 
				|| (ucs_x_bom != __ULE_UCS_BOM && charSet == UNI))
			{
				// current encoding setting mismatched with code plate of existing file
				Close();
				return false;
			}
			
			_ofsWriter.seekp(0, ios::end);
			_ofsWriter.clear();
		}
		else
			_ofsWriter.open (cFileNamePtr, ios::out|ios::binary|ios::ate);
	}

	if (! _ofsWriter)
	{
		Close();
		return false;
	}

	_characterSet =  charSet;

	// if file is unicode output code 0xfeff
	if (_characterSet == FileWriter::UNI && bExists == false)
	{
		wchar_t ucs_x_bom = __ULE_UCS_BOM;
		Word ucs_2_bom = (Word)ucs_x_bom;
		DWord ucs_4_bom = (DWord)ucs_x_bom;

		Byte* ucs_x_bom_ptr = ( (__ULE_UCS_SIZE == __ULE_UCS_2) ? (Byte*)(&ucs_2_bom) : (Byte*)(&ucs_4_bom) );

		_ofsWriter.write((char*)ucs_x_bom_ptr, __ULE_UCS_SIZE);
	}

	_bFileOpened = true;
	return true;
}

bool FileWriter::DoesExist(const char* cFileNamePtr)
{
	ifstream iTestFile;
	iTestFile.open (cFileNamePtr);
	bool bRetStatus = ((iTestFile) ? true : false);
	iTestFile.close ();
	return bRetStatus;
}

// writes ascii character string in ascii file
bool FileWriter::Write (string sBuffer)
{
	// check that file is open
	if (!_bFileOpened)
		return false;

	// check if file was opened as ascii file
	if (_characterSet != FileWriter::ASCII)
		return false;	// file was not opened as ascii file

	int n = sBuffer.length();
	for (int i = 0; i < n; i++)
		_ofsWriter.write((char*)&sBuffer[i], 1);

	return true;
}

// writes unicode character string in unicode file
bool FileWriter::Write (wstring wsBuffer)
{
	// check that file is open
	if (!_bFileOpened)
		return false;

	// check if file was opened as unicode file
	if (_characterSet != FileWriter::UNI)
		return false;	// file was not opened as unicode file

	Word wc_2;
	DWord wc_4;
	wchar_t wc_x;
	Byte* wc_ptr = NULL;

	int n = wsBuffer.length();
	for (int i = 0; i < n; i++)
	{
		wc_x = wsBuffer[i];
		wc_2 = (Word)wc_x;
		wc_4 = (DWord)wc_x;

		wc_ptr = ( (__ULE_UCS_SIZE == __ULE_UCS_2) ? (Byte*)(&wc_2) : (Byte*)(&wc_4) );

		_ofsWriter.write((char*)wc_ptr, __ULE_UCS_SIZE);
	}

	return true;
}

// write a chunk of specified size (num of bytes) in ascii file
bool FileWriter::Write (char * buf, int size)
{
	// check that file is open
	if (!_bFileOpened)
		return false;

	// check if file is end
	if (_ofsWriter.eof () || buf == NULL || size <= 0)
		return false;

	_ofsWriter.write ((char*)buf, size);

	return true;
}

// write a chunk of specified num of chars in unicode file
bool FileWriter::Write (wchar_t * buf, int n)
{
	// check that file is open
	if (!_bFileOpened)
		return false;

	// check if file was opened as unicode file
	if (_characterSet != FileWriter::UNI)
		return false;	// file was not opened as unicode file

	// check passed args
	if (buf == NULL || n <= 0)
		return false;

	Word wc_2;
	DWord wc_4;
	wchar_t wc_x;
	Byte* wc_ptr = NULL;

	for (int i = 0; i < n; i++)
	{
		wc_x = buf[i];
		wc_2 = (Word)wc_x;
		wc_4 = (DWord)wc_x;

		wc_ptr = ( (__ULE_UCS_SIZE == __ULE_UCS_2) ? (Byte*)(&wc_2) : (Byte*)(&wc_4) );

		_ofsWriter.write((char*)wc_ptr, __ULE_UCS_SIZE);
	}

	return true;
}

// moves arround given number of LOGICAL characters, NOT BYTES
// i.e. in case of Unicode, it moves arround unicode_char_size*pos bytes
void FileWriter::Seekp (long pos, short dir)
{
	// check that file is open
	if (!_bFileOpened)
		return;

	if (_characterSet == UNI)
		pos += __ULE_UCS_SIZE;

	if (_bFileOpened)
	{
		if (dir == ios::beg)
			_ofsWriter.seekp (pos, ios::beg);
		else if (dir == ios::end)
			_ofsWriter.seekp (pos, ios::end);
		else if (dir == ios::cur)
			_ofsWriter.seekp (pos, ios::cur);
	}
}

void FileWriter::Flush ()
{
	// check that file is open
	if (_bFileOpened)
		_ofsWriter.flush();
	return;
}

// tells a current BYTE location of file pointer
long FileWriter::Tellp ()
{
	// check that file is open
	if (_bFileOpened)
		return _ofsWriter.tellp();
	else
		return -1;
}

// closes file if opened
void FileWriter::Close ()
{
	// check that file is open
	if (_bFileOpened)
	{
		_ofsWriter.close ();
		_ofsWriter.clear ();
		_bFileOpened = false;
		_bApp = false;
	}
}

