#include "iostream.h"

#include "PostListResources.h"


PostListResources::PostListResources()
{
	bigramMatrix = NULL;
	unigram =NULL;

	vBigramPL = NULL;
	vUnigramPL = NULL;
	
	numberOfChars = 0;

}


bool PostListResources::Initialize(HashTable &htable,UniChar2IndexMap &cmap)
{
	
	int i,j,wordlen, hashwordcount;
	int index1,index2;


	HashNode tnode;
	wstring tword;


	numberOfChars = cmap.GetTotalNumOfChars();
	
	bigramMatrix = new RecordPtr*[numberOfChars];
	for(i=0; i<numberOfChars; i++)
		bigramMatrix[i] = new RecordPtr[numberOfChars];


	vBigramPL = new vector<int>*[numberOfChars];
	for(i=0;i<numberOfChars;i++)
		vBigramPL[i] = new vector<int>[numberOfChars]; 

	
	unigram = new RecordPtr[numberOfChars];
	vUnigramPL = new vector<int>[numberOfChars]; 


	hashwordcount = htable.elementCount();

	for(i=0; i<hashwordcount; i++)
	{
		htable.GetNodeAtIndex(i,tnode);

		tword = tnode.HashKey;
		

		
		wordlen = tword.length();

	//	cout<<"\n\n";
		//if word length is greater than and equal to 3 Build Bigram
		if(wordlen>=3)
		{
			for(j=0; j<wordlen-1;j++)
			{
				
				index1=cmap.GetIndex(tword[j]);
				index2=cmap.GetIndex(tword[j+1]);

				if(index1==UNDEF_CHAR||index2==UNDEF_CHAR)	continue;

				vBigramPL[index1][index2].push_back(i);
				

			//	cout<<j<<"\t"<<index1<<"\t"<<i<<"\n";

			}
		}
		////if word length is less than and equal to 3 Build unigram
		if (wordlen<=3)
		{
			for(j=0;j<wordlen;j++)
			{
				index1 = cmap.GetIndex(tword[j]);

				if(index1==UNDEF_CHAR) continue;

				vUnigramPL[index1].push_back(i);
			}
		}
	}
	
	
	DWord offset=0;
	for(i=0; i<numberOfChars;i++)
		for(j=0; j<numberOfChars; j++)
		{
			if(vBigramPL[i][j].size()>0)
			{
				bigramMatrix[i][j].count = vBigramPL[i][j].size();
				bigramMatrix[i][j].offset = offset;
				offset += vBigramPL[i][j].size()*sizeof(int);
			}
		}

	offset =0;
	for(i=0; i<numberOfChars;i++)
	{
		if(vUnigramPL[i].size()>0)
		{
			unigram[i].count = vUnigramPL[i].size();
			unigram[i].offset = offset;
			offset += vUnigramPL[i].size()*sizeof(int);
		}
	}
		

	return true;
}


bool PostListResources::Save(ConfigManager &config)
{
	SaveBigram(config.bigramMatrixFilePath,config.bigramPLFilePath);
	
	SaveUnigram(config.unigramListsFilePath, config.unigramPLFilePath);

	return true;
}



bool PostListResources::SaveBigram(string bigramMatrixFilename, string bigramPLFilename)
{

	FileWriter bigramWriter, bigramPLWriter;
	int i,j,k,plsize;

	try{
		bigramWriter.Open(bigramMatrixFilename.c_str(), FileWriter::UNI);
		bigramPLWriter.Open(bigramPLFilename.c_str(),FileWriter::UNI);
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	try{
		
		//write Bigram Matrix and Bigram PostLists	
		for(i=0; i<numberOfChars; i++)
			for(j=0; j<numberOfChars; j++)
			{
				//save Bigram
				bigramWriter.Write((char*)&bigramMatrix[i][j].offset,sizeof(DWord));
				bigramWriter.Write((char*)&bigramMatrix[i][j].count,sizeof(Word));

				//save Bigram PostLists
				plsize = vBigramPL[i][j].size();

				if(plsize>0)
				{
				//	cout<<"Abc\n\n";
					for(k=0;k<plsize;k++)
					{
						bigramPLWriter.Write((char*)&vBigramPL[i][j].at(k),sizeof(int));

					//	cout<<"\n"<<i<<"\t"<<j<<"\t"<<vBigramPL[i][j].at(k);
					}
				}
			}

		bigramWriter.Close();
		bigramPLWriter.Close();
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	return true;
}


bool PostListResources::SaveUnigram(string unigramListFilename, string unigramPLFilename)
{

	FileWriter unigramWriter, unigramPLWriter;
	int i,j,plsize;

	try{
		unigramWriter.Open(unigramListFilename.c_str(),FileWriter::UNI);
		unigramPLWriter.Open(unigramPLFilename.c_str(),FileWriter::UNI);
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	try{
			
		for(i=0; i<numberOfChars; i++)
		{
			unigramWriter.Write((char*)&unigram[i].offset,sizeof(DWord));
			unigramWriter.Write((char*)&unigram[i].count,sizeof(Word));

			plsize = vUnigramPL[i].size();
			if(plsize>0)
			{
				for(j=0; j<plsize ; j++)
				{
					unigramPLWriter.Write((char*)&vUnigramPL[i].at(j),sizeof(int));
				}
			}
		}
	
		unigramWriter.Close();
		unigramPLWriter.Close();
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}

	DeAllocateVBigramPL();
	DeAllocateVUnigramPL();
	return true;
}


bool PostListResources::Load(ConfigManager &config)
{

	LoadBigramMatrix(config.bigramMatrixFilePath);
	LoadUnigramList(config.unigramListsFilePath);

	return true;

}


bool PostListResources::LoadBigramMatrix(string filename)
{

	char* buf = NULL;

	try {
		if (numberOfChars <= 0)
			return false;

		// deallocate if already allocated soem memory
		DeAllocBigramMatrix();

		int i;

		// aloocate space for bigram matrix index
		this->bigramMatrix = new RecordPtr* [numberOfChars];
		for (i = 0; i < numberOfChars; i++)
			this->bigramMatrix[i] = new RecordPtr [numberOfChars];

		// open bigram matrix index file
		FileReader fr;
		fr.Open (filename.c_str(), FileReader::UNI);

		RecordPtr recPtr;
		buf = new char [numberOfChars * recPtr.sizeOf()];

		DWord offSize = sizeof (recPtr.offset);
		// read bigram martix index from file
		for (i = 0; i < numberOfChars; i++)
		{
			fr.Read (buf, numberOfChars * recPtr.sizeOf());
			char* bufPtr = buf;
			for (int j = 0; j < numberOfChars; j++)
			{
				memcpy ((void*)&(bigramMatrix[i][j].offset), bufPtr, sizeof(bigramMatrix[i][j].offset));
				memcpy ((void*)&(bigramMatrix[i][j].count), bufPtr+sizeof(bigramMatrix[i][j].offset), sizeof(bigramMatrix[i][j].count));
				bufPtr += recPtr.sizeOf();
			}

			//fr.Read ((char*)this->bigramMatrixIndex[i], numberOfChars*sizeof(RecordPtr));
		}

		delete [] buf;
		fr.Close();
	}
	catch (bad_alloc& b)
	{
		// delete locally allocated memories
		if (buf != NULL)
			delete [] buf;
		// ...

		// other alloacted meories would be deleted at class objects desctruction
		// currently just throw exception
		throw MemoryAllocationException (__FILE__, __LINE__, MEMORY_NOT_ALLOCATED, 
			"Could not allocate memory for 'Bigram Matrix Index'.");
	}
	catch (IOException& ex)
	{
		// delete locally allocated memories
		if (buf != NULL)
			delete [] buf;
		// ...

		// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Bigram Matrix Index file " +
			"could not be opened or read sucessfully.");
		throw; // re-throw the same exception object
	}

	return true;
}

bool PostListResources::LoadUnigramList(string filename)
{

	char * buf = NULL;

	try {
		if (numberOfChars <= 0)
			throw OUDLEException(__FILE__, __LINE__, 0, "Alphabet size not set");

		// deallocate if already allocated soem memory
		DeAllocUnigramList();

		// aloocate space for unigram list index
		unigram = new RecordPtr [numberOfChars];

		// open unigram matrix index file
		FileReader fr;
		fr.Open (filename.c_str(), FileReader::UNI);

		RecordPtr recPtr;
		buf = new char [numberOfChars * recPtr.sizeOf()];

		DWord offSize = sizeof (recPtr.offset);
		// read unigram list index from file
		//fr.Read ((char*)unigram, numberOfChars*sizeof(RecordPtr));
		fr.Read (buf, numberOfChars * recPtr.sizeOf());
		char* bufPtr = buf;
		for (int j = 0; j < numberOfChars; j++)
		{
			memcpy ((void*)&(unigram[j].offset), bufPtr, sizeof(unigram[j].offset));
			memcpy ((void*)&(unigram[j].count), bufPtr+sizeof(unigram[j].offset), sizeof(unigram[j].count));
			bufPtr += recPtr.sizeOf();
		}

		delete [] buf;
		fr.Close();
	}
	catch (bad_alloc& b)
	{
		// delete locally allocated memories
		if (buf != NULL)
			delete [] buf;
		// ...
		
		// alloacted meories would be deleted at class objects desctruction
		// currently just throw exception
		throw MemoryAllocationException (__FILE__, __LINE__, MEMORY_NOT_ALLOCATED, 
			"Could not allocate memory for 'Unigram List Index'.");
	}
	catch (IOException& ex)
	{
		// delete locally allocated memories
		if (buf != NULL)
			delete [] buf;
		// ...

		// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Unigram List Index file " +
			"could not be opened or read sucessfully.");
		throw; // re-throw the same exception object
	}

	return true;
}

PostListResources::~PostListResources()
{

	DeAllocBigramMatrix();
	DeAllocUnigramList();

	DeAllocateVBigramPL();
	DeAllocateVUnigramPL();
}

void PostListResources::DeAllocBigramMatrix()
{
	if (this->bigramMatrix != NULL)
	{
		for (int i = 0; i < numberOfChars; i++)
			if (this->bigramMatrix[i] != NULL)
				delete [] bigramMatrix[i];

		delete [] bigramMatrix;
		
		bigramMatrix = NULL;
	}

	return;
}

void PostListResources::DeAllocUnigramList()
{
	if (this->unigram != NULL)
		delete [] unigram;

	unigram = NULL;

	return;
}

void PostListResources::DeAllocateVBigramPL()
{
	if (vBigramPL != NULL)
	{
		for (int i = 0; i < numberOfChars; i++)
			if (this->vBigramPL[i] != NULL)
				delete [] vBigramPL[i];

		delete [] vBigramPL;
		
		vBigramPL = NULL;
	}
}

void PostListResources::DeAllocateVUnigramPL()
{
	if (this->vUnigramPL != NULL)
		delete [] vUnigramPL;

	vUnigramPL = NULL;
}


void PostListResources::SetNumberOfChars(int num)
{
	numberOfChars = num;
}



PostList::~PostList(){
	if (data != NULL)
	{
		delete [] data;
		data = NULL;
	}
	size = 0;
}