#include <iostream.h>

#include "UniChar2IndexMap.h"



// constructor
UniChar2IndexMap::UniChar2IndexMap ()
{
	numOfChars = 0;
	// initialize all code page pointers to NULL 
	for (int i = 0; i < __ULE_CHARSET_PAGE_SIZE; i++)
		CodePages[i] = NULL;
}

// copy constructor
UniChar2IndexMap::UniChar2IndexMap (const UniChar2IndexMap & r)
{
	numOfChars = r.numOfChars;
	
	for (int i = 0; i < __ULE_CHARSET_PAGE_SIZE; i++)
	{
		// ith code page exists in the right side obj
		if (r.CodePages[i] != NULL)
		{
			// allocate memeory for ith code page in left side obj
			CodePages[i] = new Word [__ULE_CHARSET_PAGE_SIZE];
			if (CodePages[i] == NULL)   // if could not allocate memory
				return;

			// copy entire array from right side object to left side obj
			memcpy ((void*)CodePages[i], (void*)r.CodePages[i], 
					__ULE_CHARSET_PAGE_SIZE*sizeof(Word));
		}
		else
			CodePages[i] = NULL;
	}
}

// destructor
UniChar2IndexMap::~UniChar2IndexMap ()
{
	Clear();
}

int UniChar2IndexMap::GetTotalNumOfChars ()
{
	return numOfChars;
}

bool UniChar2IndexMap::Initialize(string charFile)
{
	FileReader charReader;
	wstring character;


	try{
		charReader.Open(charFile.c_str(),FileReader::UNI);
	}
	catch (IOException& ex)
	{	// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file not opened sucessfully.");
		throw; // re-throw the same exception object
	}
	
	try{
		while (charReader.EoF () == false)
		{
			if(charReader.GetLine(character,2,'\n')==true)
				Insert((wchar_t)character[0]);
		}
//		cout<<"here = "<<numOfChars;

		charReader.Close();

	}
	catch (IOException& ex)
	{
		// set params telling "where" actaully it caused exception
		ex.SetFileName (__FILE__);
		ex.SetLineNum (__LINE__);
		ex.SetExceptionMsg (ex.GetExceptionMsg() + " Config file reading problem occurred.");
		throw; // re-throw the same exception object
	}

	return true;

}

// finds index of given char in O(1); returns UNDEF_CHAR if not persent in defined char set
unsigned short UniChar2IndexMap::GetIndex (wchar_t wc)
{
	Byte chHighByte;
	Byte chLowByte;

	chLowByte = char(wc);
	chHighByte = char(wc >> 8);

	if (CodePages[chHighByte] != NULL)
	{
		// return index of given char (of invalid code UNDEF_CHAR if it is not present)
		return CodePages[chHighByte][chLowByte];
	}
	
	return UNDEF_CHAR;	// code page not found for given char
}

int UniChar2IndexMap::Insert(wchar_t wc)
{
	Byte chLowByte = Byte (wc);
	Byte chHighByte = Byte (wc >> 8);

	// check code page for this char is not already created
	if (CodePages[chHighByte] == NULL)
	{
		CodePages[chHighByte] = new Word [__ULE_CHARSET_PAGE_SIZE];
		if (CodePages[chHighByte] == NULL)   // if could not allocate memory
			return false;

		// initialize all its elements with UNDEF_CHAR to mark empty
		memset ((void*)(CodePages[chHighByte]), (Byte)UNDEF_CHAR, 
				__ULE_CHARSET_PAGE_SIZE*sizeof(Word));
	}

	// insert sequential index number to cur char's loc in its corresponding code page
	CodePages[chHighByte][chLowByte] = numOfChars++;
	return 1;
}



wstring UniChar2IndexMap::Validate(wstring word)
{
	wstring tword;
	int i=0;
	
	tword.erase();
	for(i=0; i<word.length(); i++)
	{
		if(GetIndex(word[i]) != UNDEF_CHAR)
			tword += (wchar_t)word[i];
	}

	return tword;
}



 

void UniChar2IndexMap::Clear()
{
	// de-allocate space for code pages
	for (int i = 0; i < __ULE_CHARSET_PAGE_SIZE; i++)
	{
		if (CodePages[i] != NULL)
		{
			delete [] CodePages[i];
			CodePages[i] = NULL;
		}
	}

	numOfChars = 0;

	return;
}
